#include <string.h>
#include <stdio.h>
#include <ctype.h>
#include "macro.h"


extern FILE *prcs, *dbg;
extern int wc, wnb, ct, cnb;
extern char sj[], oj[], vr[], sz[], oz[], vz[], pst[], Dbg, LOG;
extern char aj[WordNumb][YakugoMax][JWordLen], az[WordNumb][YakugoMax][ZokuLen],
	en[WordNumb][SentLen], csq[WordNumb];
char caj[YakugoMax][JWordLen], caz[YakugoMax][ZokuLen];
char ptek[195], dumt[195];
//int  comma;
//extern char *dir;

tekyk(){
    FILE *tekpt;
    int i;
    int  ks, j, pwct, f;
    char c0, c1, tf, cd[3], tcd[3], Tekiyakufile[50];
//    sprintf(Tekiyakufile, "%sTEKIYAKU.HON", dir);
    strcpy(Tekiyakufile, "TEKIYAKU.HON");
    for(ks = 0; ks <= 2; ks++)
      for(wc = wnb; wc >= 0; wc--){
	if(aj[wc][1][0] == '\0') continue;
	else {
	  for(i = 0; i <= wnb; i++) csq[i] = aj[i][0][0];
	  csq[wnb+1] = '\0';
	  for(i = 0; i <= YakMaxCt; i++){
	    if(aj[wc][i][0] == '\0') break;
	    strcpy(caj[i], aj[wc][i]);	strcpy(caz[i], az[wc][i]);
	  }
	  cnb = --i;	pwct = 0;	j = 1;
	  ptek[192] = '\0';
	  while(cnb >= 1 && j <= cnb && pwct == 0){
	    c0 = caj[0][0];	j = 1;
	    while(j <= cnb){
	      if(pwct == 1 && j > 0 && c1 == caj[j][0]){ j++;	continue; }
	      c1 = caj[j][0];
	      if(c0 < c1){ cd[0] = c0;	cd[1] = c1; }
	      else { cd[0] = c1;	cd[1] = c0; }
	      cd[2] = '\0';
	      if(*pst){
		if(strcmp(cd, "11")==0 ||
		(strcmp(en[wc],"have")==0 && aj[wc+2][0][0]=='0')){
                  zenbun_comp(j, cd);
		  if(ct == 10) continue;
		  if(ct == 20) break;
		  if(ct == 30) goto NXTK;
		}
	      }
	      pwct = 0;
	      if((tekpt = fopen(Tekiyakufile, "r")) == '\0'){
		printf("Teki I[vł܂łB");	exit(1);
	      }
	      while((f=(int)fgets(ptek, 192, tekpt))){ ct = 0;
		ptek[strlen(ptek)-1] = '\0';
		tcd[0] = ptek[0];	tcd[1] = ptek[1];	tcd[2] = '\0';
		if(strcmp(cd, tcd) > 0) continue;
		if(strcmp(cd, tcd) < 0) break;
		strcpy(dumt, ptek);
		tekcd(&tf);
		if(tf != 'T') continue;
//    fprintf(prcs, "--- tekyk --- %s \n", ptek);
		yakugoSute(tcd, &j);
		if(ct != 0) break;
	      }
	      fclose(tekpt);
	      if(f != 0 && tf == 'T'){
		if(ct == 10) continue;
		if(ct == 20) break;
		if(ct == 30) goto NXTK;
	      }
	      if(*pst && strcmp(cd,"11") && strcmp(en[wc],"they") &&
	      strcmp(en[wc],"one") && strcmp(en[wc],"be") &&
	      (strcmp(en[wc],"have") || aj[wc+2][0][0] != 0)){
		zenbun_comp(j, cd);
                if(ct == 10) continue;
		if(ct == 20) break;
		if(ct == 30) goto NXTK;
	      }
	      pwct = 1;	j++;
	    }
	  }
	}
	NXTK:
	for(i = 0; i <= cnb; i++){
	  strcpy(aj[wc][i], caj[i]);	strcpy(az[wc][i], caz[i]);
 if(Dbg == 'Y') fprintf(dbg, "--- tekyk --- %s %s %d \n", ptek, caj[i], ct);
	}
	while(i < YakMaxCt){
	  aj[wc][i][0] = '\0';	az[wc][i++][0] = '\0';
	}
      }
}

tekcd(tf)
char *tf;
{
    int  comma, jtpt;
    char *pj, *dump;
    *tf = '\0';	ct = 0;
    posCk(&comma);
    if(ct == -1) return;
    dump = dumt;
    while(pj = (char*)index(dump, '=')){
      Eqcd(&jtpt, &comma, pj);
      if(ct < 1 && *pj == ' ') return;
      dump = pj+2;
    }
    dump = dumt;
    while(pj = (char*)strstr(dump, "<>")){
      Eqcd(&jtpt, &comma, pj);
      if(ct >= 0 && *pj == ' ') return;
      dump = pj+2;
    }
    dump = dumt;
    while(pj = (char*)index(dump, '=')){
      eqazck(pj);	if(ct < 1) return;
      dump = pj+1;
    }
    dump = dumt;
    while(pj = (char*)strstr(dump, "<>")){
      eqazck(pj);
	if(ct >= 0) return;
      dump = pj+1;
    }
    dump = dumt;
    while(pj = (char*)strstr(dump, "<<")){
      ineqck(pj, 'T');	if(ct == 0) return;
      *pj = ' ';	dump = pj+1;
    }
    *tf = 'T';
    //fprintf(prcs, "--- tekcd --- dumt = %s ct = %d \n", dumt, ct);
}

posCk(comma)
int  *comma;
{
    char b;
    if(strstr(ptek, "FT"))
      if(wc > 0){
	b = aj[wc-1][0][0];
	if(b != '0' && b != '?' && b != '!' && b != '|'){ ct = -1; return; }
      }
    if(strstr(ptek, "LT"))
      if(wc < wnb){
	b = aj[wc+1][0][0];
	if(b != '0' && b != '?' && b != '!' && b != '|'){ ct = -1; return; }
      }
    if(strstr(ptek, "FK")) fukuCk(comma);
    if(strstr(ptek, "PLS")){
      plrSts(comma);
      if(*comma == -1){ ct = -1;	return; }
    }
}

fukuCk(comma)
int  *comma;
{
    int  i;
    if(wc < 2){ ct = -1;	return; }
    *comma = 0;
    for(i = wc-1; i >= 1; i--){
      if(strcmp(en[i], "and") == 0 || strcmp(en[i], ",") == 0){
	*comma = i;
	if(strcmp(en[i-1], ",") == 0) (*comma)--;
 if(Dbg == 'Y') fprintf(dbg, "--- fukuCk --- comma = %d \n", *comma);
	break;
      }
    }
    if(*comma == 0){ ct = -1;	return; }
    for(i = *comma+1; i <= wnb; i++){
      if(aj[i][0][0] == '3' && (aj[i][1][0] == '\0' || aj[i][1][0] == '3'))
	break;
    }
    if(i >= wnb){ ct = -1;	return; }
    for(i = *comma-1; i >= 0; i--){
      if(aj[i][0][0] == '3' && (aj[i][1][0] == '\0' || aj[i][1][0] == '3'))
	break;
    }
    if(i==0 && (aj[i][0][0]!='3'|| aj[i][1][0]!='\0')){ ct= -1;	return; }
 if(Dbg == 'Y') fprintf(dbg, "--- fukuCk --- OK ptek = %s \n", ptek);
}

f1jt(comma, jtpt, tst)
int  comma, *jtpt;
char tst[];
{
    int  i;
    char jc, tc;
 if(Dbg == 'Y') fprintf(dbg, "--- f1jt --- comma = %d \n", comma);
    jc = tst[0];	tc = tst[1];
    for(i = comma-1; i >= 0; i--){
      if(aj[i][0][0] == jc && aj[i][1][0] == '\0' && index(az[i][0], tc)){
	*jtpt = i;	return;
      }
    }
    ct = -1;
}

wc3cal(pj, pc)
char pc, *pj;
{
    int  wc3;
    if(*(pj-2) == '-') wc3 = wc - pc + '0';
    else wc3 = wc + pc - '0';
    return(wc3);
}

Eqcd(jtpt, comma, pj)
int  *jtpt, *comma;
char *pj;
{
    int  ss, wc3, wdps, posct, negct;
    char pc, bb[45], tst[80], *ch;
    pc = *(pj-1);
    if(pc != 'A' && pc != '#' && (pc != 'Z' || *(pj-2)=='S' ||
    *(pj-2)=='O' || *(pj-2)=='V' || *(pj-3)=='S' || *(pj-3)=='O')){
      ch = (char*)index(pj+1, ';');
      if(*pj == '=') ptcpy(tst, pj+1, ch);
      else ptcpy(tst, pj+2, ch);
      ct = 0;
      sovck(pj, tst, pc);
      if(ct != 0){ *pj = ' ';	return; }
      if(pc == 'O'){
	ch = tst;
	do{
	  if(index(sz,*ch) || index(oz,*ch)){ ct=1; *pj = ' ';	return; }
	} while(*(++ch));
	ct = -1;	*pj = ' ';	return;
      }
      if(*(pj-2) == 'E' && pc == 'B'){
	if(pst == (char*)strstr(pst, tst) ||
	 (*tst == '\\' && pst == (char*)index(pst, tst[1]))){
	  ct = 1;	*pj = ' ';	return;
	}
	else{
	  ct = -1;	*pj = ' ';	return;
	}
      }
      if(*(pj-2) == 'B' && pc == 'S'){
	if(strstr(pst, tst+1)){ ct = 1;	*pj = ' ';	return; }
	else{ ct = -1;	*pj = ' ';	return; }
      }
      if(pc == 'T' && *(pj-2) == 'J'){
	ct = 1;
	f1jt(*comma, jtpt, tst);	*pj = ' ';	return;
      }
      if(pc == '1' && *(pj-3) == 'P'){
        wdps = plwd(*comma, *(pj+2)-'0');
        if(wdps >= 0) ct = 1;
        *pj = ' ';	return;
      }
      if(*(pj-3) == 'F' && *(pj-2) == '2') wc3 = *jtpt;
      else {
	if(*(pj-3) == '2' && *(pj-4) == 'P') wc3 = wdps+pc-'0';
	else wc3 = wc3cal(pj, pc);
      }
      if(wc3 < 0 || wc > wnb){ ct = -1;	*pj = ' ';	return; }
      if(*(pj-2) == 'E' || *(pj-3) == 'E' || *(pj-1) == 'E'){
	eneq(tst, wc3);
	if(ct == 0) ct = -1;
	*pj = ' ';	return;
      }
      ch = (char*)index(pj-4, 'D');
      if(ch && ch <= pj-2 || *(pj-3) == 'F' && ch < pj){
	dblck(wc3, tst, pj);	*pj = ' ';	return;
      }
      posct = 0;	negct = 0;
      if(pc == '0'){
	for(ss = 0; ss <= cnb; ss++){
	  if(*(pj-2) == 'J' || *(pj-3) == 'J'){
	    *bb = caj[ss][0];	*(bb+1) = '\0';
	  }
	  else {
	    strcpy(bb, caz[ss]);
	    if(ch = (char*)index(bb, 'y')) *(ch+1) = '\0';
	  }
	  if(*bb){ ch = tst;
	    do{
	      if(index(bb, *ch)){ ++posct;	goto NXT1; }
	      if(*bb == *ch && *(bb+1) == '\0'){ ++posct;	goto NXT1; }
	    } while(*(++ch));
	    ++negct;
	  }
	  else ++negct;
	  NXT1:;
	}
      }
      else {
	for(ss = 0; ss <= YakMaxCt; ss++){
	  if(aj[wc3][ss][0] == '\0') break;
	  if(*(pj-2) == 'J' || *(pj-3) == 'J'){
	    *bb = aj[wc3][ss][0];	*(bb+1) = '\0';
	  }
	  else {
	    strcpy(bb, az[wc3][ss]);
	    if(ch = (char*)index(bb, 'y')) *(ch+1) = '\0';
	  }
	  if(*bb){ ch = tst;
	    do{
	      if(index(bb, *ch)){ ++posct;	goto NXT2; }
	      if(*bb == *ch && *(bb+1) == '\0'){
		++posct;	goto NXT2;
	      }
	    } while(*(++ch));
	    ++negct;
	  }
	  else ++negct;
	  NXT2:;
	}
      }
      if(posct && (!negct)){
	ct = 1;	*pj = ' ';	return;
      }
      if((!posct) && negct) ct = -1;
      else ct = 0;
      *pj = ' ';
    }
}

sharpCk(pj, tst)
char tst[], *pj;
{
    int tt, ss;
    char bb[45], *ptR, *ch;
    if(*(pj+1) == 'R'){
      for(ss = wc-1; ss >= 0; ss--){
	if(*(aj[ss][0]) == '#'){
	  left(bb, az[ss][0], 2);	break;
	}
      }
      for(ss = wc-2; ss >= 0; ss--){
	if(strstr(aj[ss][0], bb)){
	  if(ptR = (char*)index(az[ss][0], 'R')){
	    ++ptR;	ch = tst;
	    while(*ch){
	      if(*ptR == *ch++){ ct = 1;	return; }
	    }
	    ct = -1;	return;
	  }
	  else{
	 printf("\n Lack of 'R' information on %s in the dictionary.\n",en[ss]);
	    exit(1);
	  }
	}
      }
      if(ss == 0)
	printf("ΏۋȂB bb = %s  aj[wc][0] = %s \n", bb, aj[wc][0]);
    }
    else{
      if(ch = (char*)strchr(aj[wc][0], '#')){
    //fprintf(prcs, "--- sharpCK --- aj[wc] = %s tst = %s \n", aj[wc], tst);
        ptcpy(bb, ch, ch+2);
        for(ss = wc+2; ss <= wnb; ss++){
          if(strstr(az[ss][0], bb)){
            for(tt = ss+1; tt <= wnb; tt++){
              if(aj[tt][0][0] == '#'){
                ct = -1;
                return;
              }
              ch = tst;
              while(*ch){
                if(strchr(az[tt][0], *ch++)){
                  ct = 1;
                  return;
	        }
              }
            }
            ct = -1;
            return;
          }
        }
      }
      else ct = -1;
    }
}

eneq(tst, wc3)
int  wc3;
char tst[];
{
    int  wc4, ln, lne;
    char ec[SentLen], estr[SentLen], dums[20], *pt, *ch, chb;
    ct = 0;	strcpy(ec, en[wc3]);
    if(strcmp(tst, "EB") == 0){
      if(*ec > 'Z') *ec -= 32;
      if(pst == (char *)strstr(pst, ec)) goto OKE;
      else return;
    }
    if(strcmp(tst, "*BS") == 0){
      strcpy(dums, " ");	strcat(dums, ec);
      if(pt = (char *)strstr(pst, dums)){
	pt += strlen(dums);	chb = *pt;
	if(chb == ' ' || chb == '.' || chb == ',') goto OKE;
	else return;
      }
      else return;
    }
    if(*tst == '/'){
      ln = strlen(ec);	lne = strlen(tst)-1;
      if(strcmp(ec+ln-lne, tst+1) == 0) goto OKE;
      else return;
    }
    if(*tst == '*'){
      if(*(tst+1) == 'E'){
	if(*(tst+2) == '-') wc4 = wc+48-*(tst+3);
	else wc4 = wc+*(tst+2)-48;
        strcpy(estr, en[wc4]);
      }
      else strcpy(estr,tst+1);
      if(strstr(ec, estr)) goto OKE;
      else return;
    }
    if(*tst == 'E'){
      if(*(tst+1) == '-') wc4 = wc-tst[2]+48;
      else wc4 = wc+tst[1]-48;
      strcpy(tst, en[wc4]);
    }
    ln = strlen(tst);	lne = strlen(ec);
    if(lne < ln) return;
    if(lne == ln){
      if(strcmp(ec, tst) != 0) return;
      else goto OKE;
    }
    if(lne > ln){
      if(ec == (char *)strstr(ec, tst) && (ec[ln] == ' ' || tst[ln-1] == ' '))
	goto OKE;
      pt = ec + lne - ln;
      if(*tst != ' ') --pt;
      if(*pt== ' ' && (ch= (char *)strstr(pt, tst)) && (ch== pt+1 || ch== pt))
	goto OKE;
      return;
    }
    OKE:	ct = 1;
}

dblck(wc3, tst, pj)
int  wc3;
char tst[], *pj;
{
    int i;
    int  ln, ln1, posct, negct;
    static char p3[3], bj[YakugoMax][JWordLen], bz[YakugoMax][ZokuLen];
    unsigned char dums[WordLen];
    char *ch;
    ct = 0;
    posct = 0;	negct = 0;
    if(wc3 == wc){
      for(i = 0; i <= cnb; i++){
	strcpy(bj[i], caj[i]);
	if(strlen(bj[i]) >= 2) cutoff(bj[i], 0, 1);
	strcpy(bz[i], caz[i]);
      }
      if(cnb < YakMaxCt) bj[cnb+1][0] = '\0';
    }
    else{
      for(i = 0; i <= YakMaxCt; i++){
	strcpy(bj[i], aj[wc3][i]);
	if(strlen(bj[i]) >= 2) cutoff(bj[i], 0, 1);
	strcpy(bz[i], az[wc3][i]);
      }
    }
    if(strlen(tst) <= 1 && *tst != '$' && (*(pj-2) == 'D' || *(pj-3) == 'D')){
      ct = 0;	return;
    }
/* printf("--- dblck --- ptek = %s  %s \n", ptek+strlen(ptek)-6, pj-4);*/
    if((*(pj-3) == 'D' && *(pj-2) == 'J') ||
    (*(pj-4) == 'D' && (*(pj-3) == 'J' || *(pj-3) == 'F'))){
      if(strcmp(tst, "SJ$") == 0){
	if(*sj == '\0'){ ct = -1;	return; }
	else strcpy(tst, sj+1);
      }
      if(strcmp(tst, "OJ$") == 0){
	if(*oj == '\0'){ ct = -1;	return; }
	else strcpy(tst, oj+1);
      }
      if(strcmp(tst, "VR$") == 0){
	if(*vr == '\0'){ ct = -1;	return; }
	else strcpy(tst, vr);
      }
      ln1 = strlen(tst);
      for(i = 0; i <= YakMaxCt; i++){
	if(bj[i][0] == '\0') goto OKE;
	ln = strlen(bj[i]);
	if(ln >= ln1){
	  if(ch = (char*)strstr(bj[i], tst)){
	    if(ch == bj[i] || ln1 == 1){ ++posct;	goto NXT1; }
	    else {
	      right(bj[i], dums, ln1);
	      if(strcmp(dums, tst) == 0){ ++posct;	goto NXT1; }
	    }
	  }
	  ++negct;
	}
	else {
	  if((ch = (char*)strstr(tst, bj[i])) && ln >= 4){
	    if(ch == tst){ ++posct;	goto NXT1; }
	    else {
	      right(tst, dums, ln);
	      if(strcmp(dums, bj[i]) == 0){ ++posct;	goto NXT1; }
	    }
	  }
	  ++negct;
	}
	NXT1:;
      }
    }
    else {
      if(*(pj-1) == 'D' || *(pj-2) == 'D' || *(pj-3) == 'D' && *(pj-2) == '-'){
	for(i = 0; i <= YakMaxCt; i++){
	  if(bj[i][0] == '\0') goto OKE;
	  ch = tst;
	  while(*ch){
	    *p3 = *ch++;	*(p3+1) = *ch++;	*(p3+2) = '\0';
	    if(strstr(bz[i], p3) != 0){ ++posct;	goto NXTD; }
	  }
	  ++negct;
	  NXTD:;
	}
      }
    }
    OKE:
/* printf("--- dblck --- ptek = %s  %d-%d  %s \n", ptek+strlen(ptek)-6, posct, negct, tst);*/
    if(posct && (!negct)) ct = 1;
    else
     if((!posct) && negct) ct = -1;
}

qzck(tst)
char tst[10];
{
    char *ptq;
    ct = 0;
    if(wc > wnb - 2) return;
    if(ptq = (char *)index(csq+wc, *tst)){
      if(ptq == (char *)strstr(ptq, tst)) ct = 1;

    }
}

eqazck(pj)
char *pj;
{
    int  i, j, j3, j9, j33, j00, tsta, tend, posct, negct;
    char pc, p3, tst[30], cq1[WordNumb], bb[45], eu[3], *ch;
    ct = 0;	pc = *(pj-1);	ch = (char*)index(pj+1, ';');
    if(*pj == '=') ptcpy(tst, pj+1, ch);
    else ptcpy(tst, pj+2, ch);
    if((pc=='A' && wc==0) || (pc=='Z' && wc==wnb && aj[wc-1][0][0] !='#')){
      ct = -1;	goto FALS;
    }
    if(*(pj-2) == 'Q' && pc == 'Z'){
      qzck(tst);
      if(ct == 1) goto OKEZ;
      else{ ct = -1;	goto FALS; }
    }
    if(pc == '#'){
      ct = 0;	sharpCk(pj, tst);
      if(ct == 1) goto OKEZ;
      if(ct == -1) goto FALS;
    }
    left(eu, en[wc-1], 2);
    if(pc == 'A'){
      tsta = wc-1;	tend = 0;
      if(eu[0] == '#' && wc > 3)
	for(i = wc-3; i >= 0; i--)
	  if(strstr(aj[i][0], eu)){ tsta = i-1;	break; }
      left(cq1, csq, wc);
      j9 = instrc(cq1, '9');	j9=0; // 15/12/24 ڑ(and,or,..)
      j3 = instrc(csq, '3');
      j33 = ninstrc(j9+1, csq, '3');
      if(j3 > 0 && j9 > j3 && j33 > 0){
	if(j9 == j3+1 && j33 == j9+1 && en[j3-1] == en[j9]) tend = 0;
	else tend = j9-1;
      }
 // if(Dbg == 'Y') fprintf(dbg, "--- eqazck --- tst = %s tend = %d \n", tst, tend);
    }
    else {
      tend = wc+1;	tsta = wnb;
      if(eu[0] == '#' && wc > 3){
	for(i = wc-3; i >= 0; i--)
	  if(strstr(aj[i][0], eu)){ tend = i+1;	break; }
	for(i = 0; i < wc; i++)
	  if(aj[i][0][0] == '#'){ tsta = i-1;	break; }
      }
      right(csq, cq1, strlen(csq)-wc-1);
      j9 = instrc(cq1, '9');  j9 = 0; // 15/12/24 ڑ(and,or,..)
      j00 = ninstrc(j9+1, cq1, '0');
      j3 = instrc(csq, '3');
      j33 = ninstrc(j9+1, cq1, '3');
      if(j9 > 0 && j3 > 0 && wc+1+j9 > j3 && j33 > 0 && 
      (j00 == 0 || (!index(cq1,'|')) || j33 < j00)) tsta = wc+j9;
      for(i = tend; i <= tsta; i++){
	p3 = csq[i];
	if(p3== '\0' || p3== '0' || p3== '|' || p3=='?' || p3=='!' || p3=='#')
	  break;
      }
      if(i < tsta) tsta = i;
    }
    if(tsta < tend) ct = -1;
    for(i = tsta; i >= tend; i--){
      if(*(pj-2) == 'D' || *(pj-3) == 'D'){
	dblck(i, tst, pj);
	if(ct == 1) goto OKEZ;
	else{
	  if(*pj == '<' && ct == 0) goto FALS;
	  goto NXT3;
	}
      }
      else{
	if(*(pj-2) == 'E'){
 // if(Dbg == 'Y') fprintf(dbg, "--- eqazck --- tst = %s cq1= %s j9= %d \n", tst, cq1, j9);
	  eneq(tst, i);
	  if(ct == 1) goto OKEZ;
	  else{ ct = -1;	goto NXT3; }
	}
	else{
	  posct = 0;	negct = 0;
	  for(j = 0; j <= YakMaxCt; j++){
	    if(aj[i][j][0] == '\0') break;
	    if(*(pj-2) == 'J' || *(pj-3) == 'J') left(bb, aj[i][j], 1);
	    else{
	      if(az[i][j][0] == '\0'){ ++negct;	continue; }
	      else{
		strcpy(bb, az[i][j]);
		if(ch = (char*)index(bb, 'y')) *(ch+1) = '\0';
	      }
	    }
	    ch = tst;
	    while(*ch){
	      if(index(bb, *ch++)){ ++posct;	goto NXT1; }
	    }
	    ++negct;
	    NXT1:;
	  }
	  if(posct && (!negct)){
	    ct = 1;	goto OKEZ;
	  }
	  if((!posct) && negct && (i == tsta || ct == -1)) ct = -1;
	  else ct = 0;
	  if(*pj == '<' && ct == 0) goto FALS;
	}
      }
      NXT3:
      if(i < tsta || pc == 'A'){
	p3 = csq[i];
	if(p3 == '\0' || p3 == '0' ||
	(p3== '|' && strcmp(en[i], "f") !=0) || p3 == '?' || p3 == '!') break;
      }
    }
    goto FALS;
    OKEZ:	ct = 1;
    FALS:	if(ct) *pj = ' ';
}

zenbun_comp(j, cd)
int  j;
char cd[];
{
    int  i;
    int  yn0, yn1, lns, lno, lnv, lnm, mrk[YakugoMax];
    char md[JWordLen];
    yn0 = 0;
    yn1 = 0;
    lns = strlen(sj);
    lno = strlen(oj);
    lnv = strlen(vr);
    for(i = 0; i <= cnb; i++){
      mrk[i] = 2;
      if(index(cd, caj[i][0]) && strlen(caj[i]) >= 2){
        strcpy(md, caj[i]+1);
        if((*sj && instr(sj, md) == 2 && (!strstr(sj, "i")) ) || (*oj && instr(oj, md) == 2 && (!strstr(oj, "i")) ) || (*vr && instr(md, vr) == 1 && (!strstr(vr, "i")) ) ){
          yn0 = 1;
          mrk[i] = 1;
          continue;
        }
        lnm =strlen(md);
        if(lns > lnm && strcmp(sj+lns-lnm, md) == 0){
          yn0 = 1;
          mrk[i] = 1;
          continue;
        }
        if(lno > lnm && strcmp(oj+lno-lnm, md) == 0){
          yn0 = 1;
          mrk[i] = 1;
          continue;
        }
        if(lnv > lnm && strcmp(vr+lnv-lnm, md) == 0){
          yn0 = 1;
          mrk[i] = 1;
          continue;
        }
        if(lnv > 0 && lnm > lnv && strcmp(vr, "ł") && (!strcmp(md+lnm -lnv, vr)))
          if(strcmp(vr, "") || lnm == 6){
            yn0 = 1;
            mrk[i] = 1;
            continue;
          }
        mrk[i] = 0; 
      }
    }
    if(yn0){
      ct = 30;
      for(i = 0; i <= cnb; i++){
	if(mrk[i] == 0){
	  caj[i][0] = '\0';	yn1 = 1;
	  if(i == 0) ct = 20;
	  else {
	    if(ct == 30){
	      if(i < cnb) ct = 10;
	      if(j < i) ct = 0;
	    }
	  }
	}
      }
    }
    if(yn1 == 1) yakAkitume();
    else ct = 0;
}

sovck(pj, tst, pc)
char pc, tst[], *pj;
{
    char sov[JWordLen], *ch;
    if((*(pj-2) == 'S' && pc != '0') || *(pj-3) == 'S'){
      ct = 2;
      if(pc == 'J') strcpy(sov, sj);
      else strcpy(sov, sz);
    }
    if(*(pj-2) == 'O' || *(pj-3) == 'O'){
      ct = 2;
      if(pc == 'J') strcpy(sov, oj);
      else strcpy(sov, oz);
    }
    if(*(pj-2) == 'V'){
      ct = 2;
      if(pc == 'R') strcpy(sov, vr);
      else strcpy(sov, vz);
    }
    if(ct != 2) return;
    if(pc == 'Z' && *sov == 'b' && (ch = (char*)index(sov+2, 'b')))
      ptptcut(sov, ch);
    if(pc == 'J' || pc == 'R' || *(pj-2) == 'D'){
      if(strstr(sov, tst)) ct = 1;
      else ct = -1;
      return;
    }
    if(pc == 'Z'){
 if(Dbg == 'Y') fprintf(dbg, "--- sovck --- ptek = %s  sov = %s  tst = %s \n", ptek,sov,tst);
      do{
	if(index(sov, *tst)){
	  ct = 1;	return;
	}
      } while(*(++tst));
      ct = -1;
    }
}

yakugoSute(tcd, j)
int  *j;
char tcd[];
{
    int  i, ln, shsw, yn0, yn1;
    static int mrk[YakugoMax];
    char jz, fc, sg[WordLen], v0[JWordLen], *pch;
    pch = (char*)index(ptek+4, ';');
    ptcpy(sg, ptek+3, pch);
    shsw = 0;	jz = *(pch+1);	ct = 0;
    if(*sg == '*'){ shsw = 1;	ptcut(sg, 1); }
  //fprintf(prcs, "--- yakugoSute --- ptek = %s jz = %c \n", ptek, jz);
    if(jz == '0'){
//      if(Dbg == 'Y' || Dbg == 'y')
//	fprintf(dbg, "\n %s \n     %s ; %s \n", ptek, caj[0], caj[*j]);
//      else
      if(LOG == 'Y') fprintf(prcs, "\n %s \n     %s ; %s \n", ptek, caj[0], caj[*j]);
      if((*sg == *tcd && shsw == 0) || (*sg != *tcd && shsw)) fc = *tcd;
      else fc = tcd[1];
      ct = 30;
      for(i = 0; i <= cnb; i++){
	if(fc == caj[i][0]){ caj[i][0] = '\0';
	  if(i == 0) ct = 20;
	  else{
	    if(ct == 30){
	      if(i < cnb) ct = 10;
	      if(*j < i) ct = 0;
	    }
	  }
	}
      }
    }
    if(jz == '0') goto SUTE;
    yn0 = 0;	yn1 = 0;
    if(jz == '2'){
      ln = strlen(sg)-1;
      for(i = 0; i <= cnb; i++){
	mrk[i] = 2;	strcpy(v0, caj[i]);
	if(index(tcd, *v0)){
	  if(instr(v0, sg) == 2 ||
	  (strstr(v0, sg) && instr(v0,sg) == strlen(v0)-ln)){
/*	  if(strncmp(v0+1, sg, strlen(sg)) == 0){*/
	    yn0 = 1;	mrk[i] = 0;
	  }
	  else mrk[i] = 1;
	}
   //fprintf(prcs, "--- yakugoSute --- %s %s mrk[%d] = %d \n", sg, v0, i, mrk[i]);
      }
    }
    if(jz == '2') goto SuteMark;
    if(jz == '1'){
      for(i = 0; i <= cnb; i++){
	mrk[i] = 2;
	if(*tcd == caj[i][0]){
	  if(index(caz[i], *sg)){ yn0 = 1;	mrk[i] = 0; }
	  else mrk[i] = 1;
	}
/* if(Dbg == 'Y') printf("-- yakugoSute -- %s %s mrk[%d] = %d \n", sg, caz[i], i, mrk[i]);*/
      }
    }
    SuteMark:
    if(yn0){
//      if(Dbg == 'Y' || Dbg == 'y')
//	fprintf(dbg, "\n %s \n     %s ; %s \n", ptek, caj[0], caj[*j]);
//      else
      if(LOG == 'Y') fprintf(prcs, "\n %s \n     %s ; %s \n", ptek, caj[0], caj[*j]);
      ct = 30;
      for(i = 0; i <= cnb; i++){
	if(mrk[i] == shsw){
	  caj[i][0] = '\0';	yn1 = 1;
	  if(i == 0) ct = 20;
	  else {
	    if(ct == 30) {
	      if(i < cnb) ct = 10;
	      if(*j < i) ct = 0;
	    }
	  }
	}
      }
    }
    SUTE:
    if(jz == '0' || yn1 == 1){
      yakAkitume();
      if(cnb < 0){
	i = 0;
	while(aj[wc][i][0]){
	  strcpy(caj[i], aj[wc][i]);	strcpy(caz[i], az[wc][i]);	i++;
	}
	cnb = --i;
	if(!strstr(caj[0], "(SŁI ptek = ...")){
	  strcat(caj[0], "(SŁI ptek = ...");
	  ln = strlen(ptek);
	  if(ln <= 8) strcat(caj[0], ptek);
	  else strcat(caj[0], ptek+ln-8);
	  strcat(caj[0], ")");
	}
	ct = 0;
      }
    }
    else ct = 0;
}
